'''
My UserAgentRotator... because another libriries get old user-agent


Example:
```python 

# get Random User-Agent
UaRandom.web()
# get User-Agent like Chrome
UaRandom.web("chrome") 
# get User-Agent like Firefox
UaRandom.web("firefox")
# get User-Agent like Safari
UaRadom.web("safari")

'''
from .ua_random import UaRandom
